@@ -70,17 +70,22 @@ __all__ = [
     "RemoveTextLetterByLetter",
     "ShowSubmobjectsOneByOne",
     "AddTextWordByWord",
+    "TypeWithCursor",
+    "UntypeWithCursor",
 ]
 
 
 import itertools as it
-from typing import TYPE_CHECKING, Callable, Iterable, Sequence
+from collections.abc import Callable, Iterable, Sequence
+from typing import TYPE_CHECKING
 
 import numpy as np
 
 if TYPE_CHECKING:
     from manim.mobject.text.text_mobject import Text
+    from manim.scene.scene import Scene
 
+from manim.constants import RIGHT, TAU
 from manim.mobject.opengl.opengl_surface import OpenGLSurface
 from manim.mobject.opengl.opengl_vectorized_mobject import OpenGLVMobject
 from manim.utils.color import ManimColor
@@ -88,7 +93,6 @@ from manim.utils.color import ManimColor
 from .. import config
 from ..animation.animation import Animation
 from ..animation.composition import Succession
-from ..constants import TAU
 from ..mobject.mobject import Group, Mobject
 from ..mobject.types.vectorized_mobject import VMobject
 from ..utils.bezier import integer_interpolate
@@ -116,7 +120,7 @@ class ShowPartial(Animation):
     ):
         pointwise = getattr(mobject, "pointwise_become_partial", None)
         if not callable(pointwise):
-            raise NotImplementedError("This animation is not defined for this Mobject.")
+            raise TypeError(f"{self.__class__.__name__} only works for VMobjects.")
         super().__init__(mobject, **kwargs)
 
     def interpolate_submobject(
@@ -129,7 +133,7 @@ class ShowPartial(Animation):
             starting_submobject, *self._get_bounds(alpha)
         )
 
-    def _get_bounds(self, alpha: float) -> None:
+    def _get_bounds(self, alpha: float) -> tuple[float, float]:
         raise NotImplementedError("Please use Create or ShowPassingFlash")
 
 
@@ -169,7 +173,7 @@ class Create(ShowPartial):
     ) -> None:
         super().__init__(mobject, lag_ratio=lag_ratio, introducer=introducer, **kwargs)
 
-    def _get_bounds(self, alpha: float) -> tuple[int, float]:
+    def _get_bounds(self, alpha: float) -> tuple[float, float]:
         return (0, alpha)
 
 
@@ -225,8 +229,6 @@ class DrawBorderThenFill(Animation):
         rate_func: Callable[[float], float] = double_smooth,
         stroke_width: float = 2,
         stroke_color: str = None,
-        draw_border_animation_config: dict = {},  # what does this dict accept?
-        fill_animation_config: dict = {},
         introducer: bool = True,
         **kwargs,
     ) -> None:
@@ -240,13 +242,13 @@ class DrawBorderThenFill(Animation):
         )
         self.stroke_width = stroke_width
         self.stroke_color = stroke_color
-        self.draw_border_animation_config = draw_border_animation_config
-        self.fill_animation_config = fill_animation_config
         self.outline = self.get_outline()
 
     def _typecheck_input(self, vmobject: VMobject | OpenGLVMobject) -> None:
         if not isinstance(vmobject, (VMobject, OpenGLVMobject)):
-            raise TypeError("DrawBorderThenFill only works for vectorized Mobjects")
+            raise TypeError(
+                f"{self.__class__.__name__} only works for vectorized Mobjects"
+            )
 
     def begin(self) -> None:
         self.outline = self.get_outline()
@@ -277,7 +279,7 @@ class DrawBorderThenFill(Animation):
         alpha: float,
     ) -> None:  # Fixme: not matching the parent class? What is outline doing here?
         index: int
-        subalpha: int
+        subalpha: float
         index, subalpha = integer_interpolate(0, 2, alpha)
         if index == 0:
             submobject.pointwise_become_partial(outline, 0, subalpha)
@@ -347,10 +349,7 @@ class Write(DrawBorderThenFill):
     ) -> tuple[float, float]:
         length = len(vmobject.family_members_with_points())
         if run_time is None:
-            if length < 15:
-                run_time = 1
-            else:
-                run_time = 2
+            run_time = 1 if length < 15 else 2
         if lag_ratio is None:
             lag_ratio = min(4.0 / max(1.0, length), 0.2)
         return run_time, lag_ratio
@@ -473,7 +472,7 @@ class SpiralIn(Animation):
 
     def interpolate_mobject(self, alpha: float) -> None:
         alpha = self.rate_func(alpha)
-        for original_shape, shape in zip(self.shapes, self.mobject):
+        for original_shape, shape in zip(self.shapes, self.mobject, strict=False):
             shape.restore()
             fill_opacity = original_shape.get_fill_opacity()
             stroke_opacity = original_shape.get_stroke_opacity()
@@ -674,3 +673,176 @@ class AddTextWordByWord(Succession):
             )
         )
         super().__init__(*anims, **kwargs)
+
+
+class TypeWithCursor(AddTextLetterByLetter):
+    """Similar to :class:`~.AddTextLetterByLetter` , but with an additional cursor mobject at the end.
+
+    Parameters
+    ----------
+    time_per_char
+        Frequency of appearance of the letters.
+    cursor
+        :class:`~.Mobject` shown after the last added letter.
+    buff
+        Controls how far away the cursor is to the right of the last added letter.
+    keep_cursor_y
+        If ``True``, the cursor's y-coordinate is set to the center of the ``Text`` and remains the same throughout the animation. Otherwise, it is set to the center of the last added letter.
+    leave_cursor_on
+        Whether to show the cursor after the animation.
+
+    .. tip::
+        This is currently only possible for class:`~.Text` and not for class:`~.MathTex`.
+
+
+    Examples
+    --------
+
+    .. manim:: InsertingTextExample
+        :ref_classes: Blink
+
+        class InsertingTextExample(Scene):
+            def construct(self):
+                text = Text("Inserting", color=PURPLE).scale(1.5).to_edge(LEFT)
+                cursor = Rectangle(
+                    color = GREY_A,
+                    fill_color = GREY_A,
+                    fill_opacity = 1.0,
+                    height = 1.1,
+                    width = 0.5,
+                ).move_to(text[0]) # Position the cursor
+
+                self.play(TypeWithCursor(text, cursor))
+                self.play(Blink(cursor, blinks=2))
+
+    """
+
+    def __init__(
+        self,
+        text: Text,
+        cursor: Mobject,
+        buff: float = 0.1,
+        keep_cursor_y: bool = True,
+        leave_cursor_on: bool = True,
+        time_per_char: float = 0.1,
+        reverse_rate_function=False,
+        introducer=True,
+        **kwargs,
+    ) -> None:
+        self.cursor = cursor
+        self.buff = buff
+        self.keep_cursor_y = keep_cursor_y
+        self.leave_cursor_on = leave_cursor_on
+        super().__init__(
+            text,
+            time_per_char=time_per_char,
+            reverse_rate_function=reverse_rate_function,
+            introducer=introducer,
+            **kwargs,
+        )
+
+    def begin(self) -> None:
+        self.y_cursor = self.cursor.get_y()
+        self.cursor.initial_position = self.mobject.get_center()
+        if self.keep_cursor_y:
+            self.cursor.set_y(self.y_cursor)
+
+        self.cursor.set_opacity(0)
+        self.mobject.add(self.cursor)
+        super().begin()
+
+    def finish(self) -> None:
+        if self.leave_cursor_on:
+            self.cursor.set_opacity(1)
+        else:
+            self.cursor.set_opacity(0)
+            self.mobject.remove(self.cursor)
+        super().finish()
+
+    def clean_up_from_scene(self, scene: Scene) -> None:
+        if not self.leave_cursor_on:
+            scene.remove(self.cursor)
+        super().clean_up_from_scene(scene)
+
+    def update_submobject_list(self, index: int) -> None:
+        for mobj in self.all_submobs[:index]:
+            mobj.set_opacity(1)
+
+        for mobj in self.all_submobs[index:]:
+            mobj.set_opacity(0)
+
+        if index != 0:
+            self.cursor.next_to(
+                self.all_submobs[index - 1], RIGHT, buff=self.buff
+            ).set_y(self.cursor.initial_position[1])
+        else:
+            self.cursor.move_to(self.all_submobs[0]).set_y(
+                self.cursor.initial_position[1]
+            )
+
+        if self.keep_cursor_y:
+            self.cursor.set_y(self.y_cursor)
+        self.cursor.set_opacity(1)
+
+
+class UntypeWithCursor(TypeWithCursor):
+    """Similar to :class:`~.RemoveTextLetterByLetter` , but with an additional cursor mobject at the end.
+
+    Parameters
+    ----------
+    time_per_char
+        Frequency of appearance of the letters.
+    cursor
+        :class:`~.Mobject` shown after the last added letter.
+    buff
+        Controls how far away the cursor is to the right of the last added letter.
+    keep_cursor_y
+        If ``True``, the cursor's y-coordinate is set to the center of the ``Text`` and remains the same throughout the animation. Otherwise, it is set to the center of the last added letter.
+    leave_cursor_on
+        Whether to show the cursor after the animation.
+
+    .. tip::
+        This is currently only possible for class:`~.Text` and not for class:`~.MathTex`.
+
+
+    Examples
+    --------
+
+    .. manim:: DeletingTextExample
+        :ref_classes: Blink
+
+        class DeletingTextExample(Scene):
+            def construct(self):
+                text = Text("Deleting", color=PURPLE).scale(1.5).to_edge(LEFT)
+                cursor = Rectangle(
+                    color = GREY_A,
+                    fill_color = GREY_A,
+                    fill_opacity = 1.0,
+                    height = 1.1,
+                    width = 0.5,
+                ).move_to(text[0]) # Position the cursor
+
+                self.play(UntypeWithCursor(text, cursor))
+                self.play(Blink(cursor, blinks=2))
+
+    """
+
+    def __init__(
+        self,
+        text: Text,
+        cursor: VMobject | None = None,
+        time_per_char: float = 0.1,
+        reverse_rate_function=True,
+        introducer=False,
+        remover=True,
+        **kwargs,
+    ) -> None:
+        super().__init__(
+            text,
+            cursor=cursor,
+            time_per_char=time_per_char,
+            reverse_rate_function=reverse_rate_function,
+            introducer=introducer,
+            remover=remover,
+            **kwargs,
+        )
